import sys
import os
import torch
import fire
from opendr.engine.datasets import PointCloudsDatasetIterator
from opendr.perception.object_detection_3d import VoxelObjectDetection3DLearner
from opendr.perception.object_detection_3d import (
    KittiDataset,
    LabeledPointCloudsDatasetIterator,
)
import run.train_3d as det
import run.eval_tracking_3d as track



class Model:
    def __init__(
        self,
        model_name,
        train_function=det.train_vnn_from_classical_tanet,
        **kwargs,
    ) -> None:
        self.model_name = model_name
        self.train_function = train_function
        self.kwargs = kwargs
        print("self.kwargs", self.kwargs)

    def train(
        self, device
    ):

        print("Sending on", device)

        last_checkpoint_path = (
            "./run/models/" + self.model_name + "/{self.model_name}.json"
        )

        if os.path.exists(last_checkpoint_path):
            print("The model is already trained")
        else:
            return self.train_function(
                device=device,
                name=self.model_name,
                **self.kwargs,
            )
        
        return {}


def run_vnn_from_classical(id=0, gpu_capacity=1, total_devices=4, raise_on_infer_error=True):

    device_id = id % total_devices
    i = id

    training_samples = [2, 4]
    eval_samples_list = [1, 2, 3, 4]

    def create_models():
        result = []
        for init_vnn_name, init_vnn_weights in [
            ("fill01", "fill:stds:0.1:0.1"),
            ("fill1", "fill:stds:1:1"),
            ("fill001", "fill:stds:0.01:0.01"),
            ("fill01a001", "fill:stds:0.1:0.01"),
            ("xavier_uniform1", "xavier_uniform:stds:1:1"),
            ("xavier_uniform01", "xavier_uniform:stds:0.1:0.1"),
            ("xavier_normal1", "xavier_normal:stds:1:1"),
            ("xavier_normal01", "xavier_normal:stds:0.1:0.1"),
        ]:
            for single_training_samples in training_samples:
                name = f"vnnclass1{init_vnn_name}_tanet_car"
                result.append(
                        Model(
                            name,
                            samples_list=[single_training_samples],
                            eval_samples_list=eval_samples_list,
                            init_vnn_weights=init_vnn_weights,
                            classical_name="tanet_car_2",
                        ),
                )

        return result

    models = create_models()

    while i < len(models):
        model = models[i]

        try:
            result = model.train(
                device="cuda:" + str(device_id)
            )
            print(result)

        except Exception as e:
            if raise_on_infer_error:
                raise e
            else:
                print(e)

        i += gpu_capacity * total_devices


def run_vnn_from_classical_2(id=0, gpu_capacity=1, total_devices=4, raise_on_infer_error=True):

    device_id = id % total_devices
    i = id

    training_samples = [2, 4]
    eval_samples_list = [1, 2, 3, 4]

    def create_models():
        result = []
        for init_vnn_name, init_vnn_weights in [
        ("fill0001", "fill:stds:0.001:0.001"),
        ("fill00001", "fill:stds:0.0001:0.0001"),
        ("xavier_uniform001", "xavier_uniform:stds:0.01:0.01"),
        ("xavier_normal001", "xavier_normal:stds:0.01:0.01"),
        ]:
            for single_training_samples in training_samples:
                name = f"vnnclass1-2{init_vnn_name}_tanet_car"
                result.append(
                        Model(
                            name,
                            samples_list=[single_training_samples],
                            eval_samples_list=eval_samples_list,
                            init_vnn_weights=init_vnn_weights,
                            classical_name="tanet_car_2",
                        ),
                )

        return result

    models = create_models()

    while i < len(models):
        model = models[i]

        try:
            result = model.train(
                device="cuda:" + str(device_id)
            )
            print(result)

        except Exception as e:
            if raise_on_infer_error:
                raise e
            else:
                print(e)

        i += gpu_capacity * total_devices


def run_vnn_from_classical_2_full_training_steps(id=0, gpu_capacity=1, total_devices=4, raise_on_infer_error=True):

    device_id = id % total_devices
    i = id

    training_samples = [2, 4]
    eval_samples_list = [1, 2, 3, 4]

    def create_models():
        result = []
        for init_vnn_name, init_vnn_weights in [
        ("fill0001", "fill:stds:0.001:0.001"),
        ("fill00001", "fill:stds:0.0001:0.0001"),
        ("xavier_uniform001", "xavier_uniform:stds:0.01:0.01"),
        ("xavier_normal001", "xavier_normal:stds:0.01:0.01"),
        ]:
            for single_training_samples in training_samples:
                name = f"vnnclass1-2{init_vnn_name}_tanet_car"
                result.append(
                        Model(
                            name,
                            samples_list=[single_training_samples],
                            eval_samples_list=eval_samples_list,
                            init_vnn_weights=init_vnn_weights,
                            classical_name="tanet_car_2",
                            config="vnn_xyres_16.proto",
                        ),
                )

        return result

    models = create_models()

    while i < len(models):
        model = models[i]

        try:
            result = model.train(
                device="cuda:" + str(device_id)
            )
            print(result)

        except Exception as e:
            if raise_on_infer_error:
                raise e
            else:
                print(e)

        i += gpu_capacity * total_devices




if __name__ == "__main__":
    fire.Fire()
